@@ -15,11 +15,11 @@ my $path = "$Bin/template";
 
 my $mt = Text::MicroTemplate::File->new(
     include_path => [$path],
-    cache        => 2,
+    use_cache    => 2,
 );
 my $tx = Text::Xslate->new(
     path      => [$path],
-    cache_dir => $path,
+    cache_dir => '.xslate_cache',
     cache     => 2,
 );
 
