@@ -1,5 +1,5 @@
 package Text::Xslate::Syntax::TTerse;
-use Any::Moose;
+use Mouse;
 use Text::Xslate::Util qw(p any_in);
 use Scalar::Util ();
 
@@ -49,8 +49,8 @@ sub init_symbols {
     $s->set_std(\&std_if);
     $s->can_be_modifier(1);
 
-    $parser->symbol('FOREACH') ->set_std(\&std_foreach);
-    $parser->symbol('FOR')     ->set_std(\&std_foreach);
+    $parser->symbol('FOREACH') ->set_std(\&std_for);
+    $parser->symbol('FOR')     ->set_std(\&std_for);
     $parser->symbol('WHILE')   ->set_std(\&std_while);
     $parser->symbol('SWITCH')  ->set_std(\&std_switch);
     $parser->symbol('CASE')    ->set_std(\&std_case);
@@ -86,7 +86,7 @@ sub init_symbols {
 
     foreach my $id(keys %{$parser->symbol_table}) {
         if($id =~ /\A [A-Z]+ \z/xms) { # upper-cased keywords
-            $parser->make_alias($id => lc $id);
+            $parser->make_alias($id => lc $id)->set_nud(\&aliased_nud);
         }
     }
 
@@ -119,6 +119,16 @@ sub default_nud {
     );
 }
 
+# same as default_nud, except for aliased symbols
+sub aliased_nud {
+    my($parser, $symbol) = @_;
+    return $symbol->clone(
+        arity => 'name',
+        id    => lc( $symbol->id ),
+        value => $symbol->id,
+    );
+}
+
 sub nud_dollar {
     my($parser, $symbol) = @_;
     my $expr;
@@ -194,6 +204,10 @@ sub std_if {
     my($parser, $symbol, $expr) = @_;
     my $if = $symbol->clone(arity => "if");
 
+    my $is_modifier = defined $expr;
+
+    $parser->new_scope() unless $is_modifier; # whole if block
+
     my $cond = $parser->expression(0);
 
     if($symbol->id eq 'UNLESS') {
@@ -204,12 +218,17 @@ sub std_if {
     }
     $if->first($cond);
 
-    if(defined $expr) { # statement modifier
+    if($is_modifier) {
         $if->second([ $expr ]);
         return $if;
     }
 
-    $if->second( $parser->statements() );
+    # then block
+    {
+        $parser->new_scope();
+        $if->second( $parser->statements() );
+        $parser->pop_scope();
+    }
 
     my $t = $parser->token;
 
@@ -221,23 +240,38 @@ sub std_if {
 
         my $elsif = $t->clone(arity => "if");
         $elsif->first(  $parser->expression(0) );
-        $elsif->second( $parser->statements() );
+
+        {
+            $parser->new_scope();
+            $elsif->second( $parser->statements() );
+            $parser->pop_scope();
+        }
+
         $if->third([$elsif]);
         $if = $elsif;
         $t  = $parser->token;
     }
 
     if($t->id eq "ELSE") {
+        my $else_line = $t->line;
         $parser->reserve($t);
         $t = $parser->advance(); # "ELSE"
 
-        $if->third( $t->id eq "IF"
-            ? [$parser->statement()]
-            :  $parser->statements());
+        if($t->id eq "IF" and $t->line != $else_line) {
+            Carp::carp(sprintf "%s: Parsing ELSE-IF sequense as ELSIF, but it is likely to be a misuse of ELSE-IF. Please insert semicolon as ELSE; IF, or write it in the same line (around input line %d)", ref $parser, $t->line);
+        }
+
+        {
+            $parser->new_scope();
+            $if->third( $t->id eq "IF"
+                ? [$parser->statement()]
+                :  $parser->statements());
+            $parser->pop_scope();
+        }
     }
 
-
     $parser->advance("END");
+    $parser->pop_scope();
     return $top_if;
 }
 
@@ -256,7 +290,7 @@ sub std_switch {
     local $parser->{in_given} = 1;
 
     my @cases;
-    while($parser->token->id ne "END") {
+    while(!($parser->token->id eq "END" or $parser->token->id eq '(end)')) {
         push @cases, $parser->statement();
     }
     $switch->third( \@cases );
@@ -289,22 +323,36 @@ sub iterator_name {
     return 'loop'; # always 'loop'
 }
 
-sub std_foreach {
+# FOR ... IN ...; ...; END
+sub std_for {
     my($parser, $symbol) = @_;
 
     my $proc = $symbol->clone(arity => "for");
 
     my $var = $parser->token;
-    if($var->arity ne "variable") {
+    if(!any_in($var->arity, qw(variable name))) {
         $parser->_unexpected("a variable name", $var);
     }
     $parser->advance();
     $parser->advance("IN");
     $proc->first( $parser->expression(0) );
     $proc->second([$var]);
+
     $parser->new_scope();
     $parser->define_iterator($var);
+
     $proc->third( $parser->statements() );
+
+    # for-else
+    if($parser->token->id eq 'ELSE') {
+        $parser->reserve($parser->token);
+        $parser->advance();
+        my $else = $parser->statements();
+        $proc = $symbol->clone( arity => 'for_else',
+            first  => $proc,
+            second => $else,
+        );
+    }
     $parser->advance("END");
     $parser->pop_scope();
     return $proc;
@@ -554,7 +602,7 @@ sub std_filter {
     return( $proc, $parser->print($callfilter) );
 }
 
-no Any::Moose;
+no Mouse;
 __PACKAGE__->meta->make_immutable();
 __END__
 
@@ -654,6 +702,9 @@ Loop iterators are partially supported.
         [%- END -%]
     [% END %]
 
+Unlike Template-Toolkit, C<FOREACH> doesn't accept a HASH reference, so
+you must convert HASH references to ARRAY references by C<keys()>, C<values()>, or C<kv()> methods.
+
 Template-Toolkit compatible names are also supported, but the use of them is
 discouraged because they are not easy to understand:
 
@@ -663,6 +714,14 @@ discouraged because they are not easy to understand:
     loop.first # for loop.is_first
     loop.last  # for loop.is_last
 
+Loop control statements, namely C<NEXT> and C<LAST>, are also supported
+in both C<FOR> and C<WHILE> loops.
+
+    [% FOR item IN data -%]
+        [% LAST IF item == 42 -%]
+        ...
+    [% END -%]
+
 =head2 Conditional statements
 
     [% IF logical_expression %]
@@ -700,7 +759,7 @@ The C<INCLUDE> statement is supported.
     [% INCLUDE "file.tt" %]
     [% INCLUDE $var %]
 
-C<< WITH variablies >> syntax is also supported, although
+C<< WITH variables >> syntax is also supported, although
 the C<WITH> keyword is optional in Template-Toolkit:
 
     [% INCLUDE "file.tt" WITH foo = 42, bar = 3.14 %]
@@ -871,7 +930,7 @@ plugins which do not depend on the context object of Template-Toolkit.
 
 The following directives are not supported:
 C<INSERT>, C<PROCESS>, C<BLOCK> as a named blocks, C<USE> (but see above),
-C<PERL>, C<RAWPERL>, C<TRY>, C<THROW>, C<NEXT>, C<LAST>,
+C<PERL>, C<RAWPERL>, C<TRY>, C<THROW>,
 C<RETURN>, C<STOP>, C<CLEAR>, C<META>, C<TAGS>, C<DEBUG>, and C<VIEW>.
 
 Some might be supported in a future.
